@@ -120,7 +120,7 @@ static void      bookmarks_rename_button_cb (GtkWidget         *widget,
 static void      bookmarks_remove_button_cb (GtkWidget         *widget,
 					     GtkTreeView       *view);
 static void      bookmarks_ensure_valid     (void);
-static gboolean  bookmarks_read             (void);
+static gboolean  bookmarks_read             (const gchar       *filename);
 
 static gboolean  bookmarks_dup_finder       (GtkTreeModel   *model,
 					     GtkTreePath    *path,
@@ -169,7 +169,7 @@ static const GtkActionEntry popup_entries[] = {
 void
 yelp_bookmarks_init (void)
 {
-    gboolean read;
+    gchar   *filename = NULL;
 
     windows = NULL;
     actions_store = gtk_tree_store_new (4,
@@ -178,7 +178,13 @@ yelp_bookmarks_init (void)
 					G_TYPE_BOOLEAN,
 					G_TYPE_BOOLEAN);
 
-    read = bookmarks_read ();
+    filename = g_build_filename (g_get_home_dir (), ".gnome2",
+				 "yelp-bookmarks.xbel", NULL);
+
+    if (g_file_test (filename, G_FILE_TEST_EXISTS))
+	bookmarks_read (filename);
+
+    g_free (filename);
 }
 
 void
@@ -243,16 +249,18 @@ yelp_bookmarks_register (YelpWindow *window)
 static void
 bookmarks_ensure_valid (void)
 {
-	if (!gtk_tree_model_iter_has_child (GTK_TREE_MODEL (actions_store),
+	if (have_toc &&
+	    !gtk_tree_model_iter_has_child (GTK_TREE_MODEL (actions_store),
 					    &toc_iter)) {
 	    gtk_tree_store_remove (actions_store, &toc_iter);
 	    have_toc = FALSE;
 	}
-	if (!gtk_tree_model_iter_has_child (GTK_TREE_MODEL (actions_store),
+	if (have_doc &&
+	    !gtk_tree_model_iter_has_child (GTK_TREE_MODEL (actions_store),
 					    &doc_iter)) {
 	    gtk_tree_store_remove (actions_store, &doc_iter);
 	    have_doc = FALSE;
-	}
+	}	
 }
 
 static gboolean 
@@ -297,16 +305,27 @@ yelp_bookmarks_add (const gchar *uri, YelpWindow *window)
     gtk_tree_model_foreach (GTK_TREE_MODEL (actions_store),
 			    bookmarks_dup_finder, dup_uri);
     if (dup_title) {
-	GtkWidget *dialog;
+	char *escaped_title, *markup;
+
 	dialog = gtk_message_dialog_new_with_markup
 	    (GTK_WINDOW (window),
 	     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 	     GTK_MESSAGE_INFO,
 	     GTK_BUTTONS_OK,
-	     _("A bookmark titled <b>%s</b> already exists for this page."),
-	     dup_title);
+	     NULL);
+
+	escaped_title = g_markup_printf_escaped
+		("<b>%s</b>", dup_title);
+	markup = g_strdup_printf
+		(_("A bookmark titled %s already exists for this page."),
+		escaped_title);
+	gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), markup);
+	g_free (markup);
+	g_free (escaped_title);
+
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
+
 	g_free (dup_title);
 	dup_title = NULL;
 	return;
@@ -882,7 +901,6 @@ void
 yelp_bookmarks_write (void)
 {
     xmlTextWriterPtr file;
-    gint rc;
     GtkTreeIter top_iter, sub_iter;
     gboolean top_valid, sub_valid;
     gchar *filename;
@@ -897,14 +915,14 @@ yelp_bookmarks_write (void)
 	return;
     }
 
-    rc = xmlTextWriterStartDocument (file, NULL, NULL, NULL);
-    rc = xmlTextWriterStartElement (file, BAD_CAST "xbel");
-    rc = xmlTextWriterWriteAttribute(file, BAD_CAST "version",
+    xmlTextWriterStartDocument (file, NULL, NULL, NULL);
+    xmlTextWriterStartElement (file, BAD_CAST "xbel");
+    xmlTextWriterWriteAttribute(file, BAD_CAST "version",
                                      BAD_CAST "1.0");
 
-    rc = xmlTextWriterStartElement (file, BAD_CAST "info");
-    rc = xmlTextWriterStartElement (file, BAD_CAST "metadata");
-    rc = xmlTextWriterWriteAttribute(file, BAD_CAST "owner",
+    xmlTextWriterStartElement (file, BAD_CAST "info");
+    xmlTextWriterStartElement (file, BAD_CAST "metadata");
+    xmlTextWriterWriteAttribute(file, BAD_CAST "owner",
                                      BAD_CAST "http://live.gnome.org/Yelp");
     xmlTextWriterEndElement (file);
     xmlTextWriterEndElement (file);
@@ -925,15 +943,16 @@ yelp_bookmarks_write (void)
 				    COL_LABEL, &label,
 				    -1);
 		xmlTextWriterStartElement (file, BAD_CAST "bookmark");
- 		xmlTextWriterWriteAttribute (file, "href", name);
+ 		xmlTextWriterWriteAttribute (file, BAD_CAST "href", 
+					     BAD_CAST name);
 
-		rc = xmlTextWriterWriteElement (file,
-						BAD_CAST "title",
-						label);
+		xmlTextWriterWriteElement (file,
+					   BAD_CAST "title",
+					   BAD_CAST label);
 
-		rc = xmlTextWriterStartElement (file, BAD_CAST "info");
-		rc = xmlTextWriterStartElement (file, BAD_CAST "metadata");
-		rc = xmlTextWriterWriteAttribute
+		xmlTextWriterStartElement (file, BAD_CAST "info");
+		xmlTextWriterStartElement (file, BAD_CAST "metadata");
+		xmlTextWriterWriteAttribute
 		    (file, BAD_CAST "owner",
 		     BAD_CAST "http://live.gnome.org/Yelp");
 		xmlTextWriterEndElement (file); /* metadata */
@@ -951,14 +970,14 @@ yelp_bookmarks_write (void)
 					      &top_iter);
     }
 
-    rc = xmlTextWriterEndDocument(file);
+    xmlTextWriterEndDocument(file);
     xmlFreeTextWriter(file);
     g_free (filename);
     return;
 }
 
 static gboolean
-bookmarks_read (void)
+bookmarks_read (const gchar *filename)
 {
     xmlParserCtxtPtr   parser;
     xmlXPathContextPtr xpath;
@@ -966,12 +985,8 @@ bookmarks_read (void)
     xmlDocPtr          doc;
 
     gboolean  ret = TRUE;
-    gchar    *filename;
     gint      i;
 
-    filename = g_build_filename (g_get_home_dir (), ".gnome2", 
-				 "yelp-bookmarks.xbel", NULL);
-
     parser = xmlNewParserCtxt ();
     doc = xmlCtxtReadFile (parser, filename, NULL,
 			   XML_PARSE_NOBLANKS | XML_PARSE_NOCDATA  |
@@ -979,33 +994,34 @@ bookmarks_read (void)
 			   XML_PARSE_NONET    );
 
     if (doc == NULL) {
-	g_free (filename);
 	if (parser)
 		xmlFreeParserCtxt (parser);
 	return FALSE;
     }
 
     xpath = xmlXPathNewContext (doc);
-    obj = xmlXPathEvalExpression ("/xbel/bookmark", xpath);
+    obj = xmlXPathEvalExpression (BAD_CAST "/xbel/bookmark", xpath);
     for (i = 0; i < obj->nodesetval->nodeNr; i++) {
 	xmlNodePtr node = obj->nodesetval->nodeTab[i];
 	gchar *uri;
-	uri = xmlGetProp (node, "href");
+	uri = (gchar *) xmlGetProp (node, BAD_CAST "href");
 	if (uri) {
 	    xmlXPathObjectPtr title_obj;
 	    xpath->node = node;
-	    title_obj = xmlXPathEvalExpression ("string(title[1])", xpath);
+	    title_obj = xmlXPathEvalExpression (BAD_CAST "string(title[1])", 
+						xpath);
 	    xpath->node = NULL;
 
 	    if (title_obj->stringval)
-		bookmarks_add_bookmark (uri, title_obj->stringval, FALSE);
+		bookmarks_add_bookmark ((const gchar *) uri, 
+					(const gchar *) title_obj->stringval, 
+					FALSE);
 
 	    xmlXPathFreeObject (title_obj);
 	    xmlFree (uri);
 	}
     }
     
-    g_free (filename);
     if (obj)
 	xmlXPathFreeObject (obj);
     if (xpath)
