@@ -49,7 +49,6 @@
 
 #define STYLESHEET_PATH DATADIR"/yelp/xslt/"
 #define DB_STYLESHEET   STYLESHEET_PATH"db2html.xsl"
-#define DB_TITLE        STYLESHEET_PATH"db-title.xsl"
 
 #define BOOK_CHUNK_DEPTH 2
 #define ARTICLE_CHUNK_DEPTH 1
@@ -82,7 +81,6 @@ struct _DBWalker {
     gint depth;
     gint max_depth;
 
-    xsltStylesheetPtr       titleStylesheet;
 };
 
 static void           db_pager_class_init   (YelpDBPagerClass *klass);
@@ -101,7 +99,11 @@ static void            walker_walk_xml      (DBWalker         *walker);
 
 static gboolean        node_is_chunk        (DBWalker         *walker);
 static gboolean        node_is_division     (DBWalker         *walker);
-static gchar *         node_get_title       (DBWalker         *walker);
+static gchar *         node_get_title       (DBWalker         *walker,
+					     gchar            *type);
+
+xmlNodePtr             node_find_child      (xmlNodePtr parent, 
+					     gchar *child_name);
 
 static YelpPagerClass *parent_class;
 
@@ -156,9 +158,7 @@ db_pager_class_init (YelpDBPagerClass *klass)
 static void
 db_pager_init (YelpDBPager *pager)
 {
-    YelpDBPagerPriv *priv;
-
-    pager->priv = priv = YELP_DB_PAGER_GET_PRIVATE (pager);
+    pager->priv = YELP_DB_PAGER_GET_PRIVATE (pager);
 
     pager->priv->sects = NULL;
 
@@ -258,18 +258,20 @@ db_pager_parse (YelpPager *pager)
     walker->doc       = doc;
     walker->cur       = xmlDocGetRootElement (walker->doc);
 
-    walker->titleStylesheet = xsltParseStylesheetFile (DB_TITLE);
-
     if (!xmlStrcmp (xmlDocGetRootElement (doc)->name, BAD_CAST "book"))
 	priv->max_depth = walker->max_depth = BOOK_CHUNK_DEPTH;
     else
 	priv->max_depth = walker->max_depth = ARTICLE_CHUNK_DEPTH;
 
-    id = xmlGetProp (walker->cur, "id");
+    id = xmlGetProp (walker->cur, BAD_CAST "id");
     if (id)
-	priv->root_id = g_strdup (id);
-    else
+	priv->root_id = g_strdup ((gchar *) id);
+    else {
 	priv->root_id = g_strdup ("index");
+	/* add the id attribute to the root element with value "index"
+	 * so when we try to load the document later, it doesn't fail */
+	xmlNewProp (walker->cur, BAD_CAST "id", BAD_CAST "index");
+    }
 
     EVENTS_PENDING;
     CANCEL_CHECK;
@@ -277,11 +279,7 @@ db_pager_parse (YelpPager *pager)
     walker_walk_xml (walker);
 
  done:
-    if (walker) {
-	if (walker->titleStylesheet)
-	    xsltFreeStylesheet (walker->titleStylesheet);
-	g_free (walker);
-    }
+    g_free (walker);
 
     g_free (filename);
 
@@ -297,15 +295,12 @@ static gchar **
 db_pager_params (YelpPager *pager)
 {
     YelpDBPagerPriv *priv;
-    YelpDocInfo     *doc_info;
     gchar **params;
     gint params_i = 0;
     gint params_max = 20;
 
     d (g_print ("db_pager_process\n"));
 
-    doc_info = yelp_pager_get_doc_info (pager);
-
     g_return_val_if_fail (pager != NULL, FALSE);
     g_return_val_if_fail (YELP_IS_DB_PAGER (pager), FALSE);
     priv = YELP_DB_PAGER (pager)->priv;
@@ -387,7 +382,9 @@ db_pager_get_sections (YelpPager *pager)
 static void
 walker_walk_xml (DBWalker *walker)
 {
-    xmlChar     *id    = NULL;
+    static       gint autoid = 0;
+    gchar        autoidstr[20];
+    xmlChar     *id = NULL;
     xmlChar     *title = NULL;
     gchar       *old_id = NULL;
     xmlNodePtr   cur;
@@ -396,27 +393,39 @@ walker_walk_xml (DBWalker *walker)
     GtkTreeIter *old_iter = NULL;
     YelpDBPagerPriv *priv = walker->pager->priv;
 
+    /* check for the yelp:chunk-depth processing instruction and set 
+     * the max chunk depth accordingly */
     if (walker->depth == 0) {
 	cur = walker->cur;
 	for ( ; cur; cur = cur->prev)
 	    if (cur->type == XML_PI_NODE)
 		if (!xmlStrcmp (cur->name,
 				(const xmlChar *) "yelp:chunk-depth")) {
-		    gint max = atoi (cur->content);
+		    gint max = atoi ((gchar *) cur->content);
 		    if (max)
 			priv->max_depth = walker->max_depth = max;
 		    break;
 		}
     }
 
-    id = xmlGetProp (walker->cur, "id");
+    id = xmlGetProp (walker->cur, BAD_CAST "id");
     if (!id && walker->cur->parent->type == XML_DOCUMENT_NODE) {
-	id = xmlStrdup ("__yelp_toc");
+	id = xmlStrdup (BAD_CAST "__yelp_toc");
     }
 
     if (node_is_chunk (walker)) {
-	title = node_get_title (walker);
+	title = BAD_CAST node_get_title (walker, "titleabbrev");
+	if (!title)
+	    title = BAD_CAST node_get_title (walker, "title");
 
+	/* if id attribute is not present, autogenerate a
+	 * unique value, and insert it into the in-memory tree */
+	if (!id) {
+	    g_snprintf (autoidstr, 20, "_auto-gen-id-%d", ++autoid);
+	    xmlNewProp (walker->cur, BAD_CAST "id", BAD_CAST autoidstr);
+	    id = xmlGetProp (walker->cur, BAD_CAST "id"); 
+	}
+	
 	if (id) {
 	    gtk_tree_store_append (GTK_TREE_STORE (priv->sects),
 				   &iter, walker->iter);
@@ -428,7 +437,7 @@ walker_walk_xml (DBWalker *walker)
 				-1);
 
 	    old_id          = walker->page_id;
-	    walker->page_id = id;
+	    walker->page_id = (gchar *) id;
 
 	    old_iter     = walker->iter;
 
@@ -446,7 +455,7 @@ walker_walk_xml (DBWalker *walker)
 
     if (id) {
 	g_hash_table_insert (priv->frags_hash,
-			     g_strdup (id),
+			     g_strdup ((gchar *) id),
 			     g_strdup (walker->page_id));
     }
 
@@ -474,31 +483,88 @@ walker_walk_xml (DBWalker *walker)
 	xmlFree (title);
 }
 
-static gchar *
-node_get_title (DBWalker *walker)
+xmlNodePtr
+node_find_child (xmlNodePtr parent, gchar *child_name)
 {
-    gchar *title;
-    xmlDocPtr doc;
-    xmlChar *params[3];
-    xmlChar *outstr;
-    int outlen;
-
-    params[0] = "node";
-    params[1] = xmlGetNodePath (walker->cur);
-    params[2] = NULL;
-
-    doc = xsltApplyStylesheet (walker->titleStylesheet,
-			       walker->doc,
-			       (const char **)params);
-    if (doc == NULL || xsltSaveResultToString (&outstr, &outlen, doc, walker->titleStylesheet) < 0)
-	title = g_strdup (_("Unknown Section"));
-    else {
-	title = g_strdup (outstr);
-	xmlFree (outstr);
+    xmlNodePtr child = (xmlNodePtr) parent->children;
+    
+    while (child) {
+	if (!xmlStrcmp (child->name, BAD_CAST child_name))
+	    break;
+	child = child->next;
     }
+    return child;
+}
 
-    xmlFreeDoc (doc);
-    xmlFree (params[1]);
+
+static gchar *
+node_get_title (DBWalker *walker, gchar *type)
+{
+    gchar *title = NULL;
+    xmlChar *node_name = (xmlChar *) walker->cur->name;
+    xmlNodePtr child = NULL;
+    xmlNodePtr title_node = NULL;
+
+    if (xmlStrcmp (node_name, BAD_CAST "refentry")) {
+	/*refentry is special cased below */
+	title_node = node_find_child (walker->cur, type);
+	if (!title_node) {
+	    /* Gotta look for some other way of getting the title */
+	    gchar *looking_for = g_strdup_printf ("%sinfo", node_name);
+	    
+	    child = node_find_child (walker->cur, looking_for);
+	    g_free (looking_for);
+	    
+	    if (child)
+		title_node = node_find_child (child, type);		
+	}
+    } else {
+	/* Refentry */
+	xmlNodePtr refentry = NULL;
+	xmlNodePtr refmeta = NULL;
+	xmlNodePtr refnamediv = NULL;
+	xmlNodePtr child = walker->cur->children;
+	
+	while (child) {
+	    if (!xmlStrcmp (child->name, BAD_CAST "refmeta"))
+		refmeta = child;
+	    else if (!xmlStrcmp (child->name, BAD_CAST "refentryinfo"))
+		refentry = child;
+	    else if (!xmlStrcmp (child->name, BAD_CAST "refnamediv"))
+		refnamediv = child;
+	    else if (!xmlStrncmp (child->name, BAD_CAST "refsect", 7))
+		break;
+	    child = child->next;
+	    
+	}
+	if (refmeta) {
+	    xmlNodePtr tmp = node_find_child (refmeta, "refentrytitle");
+	    title_node = node_find_child (tmp, type);
+	}
+	if (!title_node && refentry) {
+	    title_node = node_find_child (refentry, type);
+	}
+	if (!title_node) {
+	    title_node = node_find_child (refnamediv, "refname");
+	}
+
+    }
+    /* Something pretty seriously screwy happened
+     * Try the fallback options
+     */
+    if (!title_node) {
+	xmlNodePtr block = node_find_child (walker->cur, "blockinfo");
+	if (block) {
+	    title_node = node_find_child (block, type);
+	}
+	if (!title_node) {
+	    xmlNodePtr obj = node_find_child (walker->cur, "objectinfo");
+	    if (obj)
+		title_node = node_find_child (obj, type);
+	}
+    }
+    
+    title = (gchar *) xmlNodeGetContent (title_node);
 
     return title;
 }
