@@ -56,49 +56,49 @@ yelp_util_split_font_string (const gchar *font_name, gchar **name, gint *size)
 static gboolean
 gecko_prefs_set_bool (const gchar *key, gboolean value)
 {
-	nsCOMPtr<nsIPrefService> prefService =
-		do_GetService (NS_PREFSERVICE_CONTRACTID);
-	nsCOMPtr<nsIPrefBranch> pref;
-	prefService->GetBranch ("", getter_AddRefs (pref));
+	nsresult rv;
+	nsCOMPtr<nsIPrefService> prefService (do_GetService (NS_PREFSERVICE_CONTRACTID, &rv));
+	NS_ENSURE_SUCCESS (rv, FALSE);
 
-	if (pref) {
-		nsresult rv = pref->SetBoolPref (key, value);
-		return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-	}
-	
-	return FALSE;
+	nsCOMPtr<nsIPrefBranch> pref;
+	rv = prefService->GetBranch ("", getter_AddRefs (pref));
+	NS_ENSURE_SUCCESS (rv, FALSE);
+
+	rv = pref->SetBoolPref (key, value);
+
+	return NS_SUCCEEDED (rv) != PR_FALSE;
 }
 
 static gboolean
 gecko_prefs_set_string (const gchar *key, const gchar *value)
 {
-	nsCOMPtr<nsIPrefService> prefService =
-		do_GetService (NS_PREFSERVICE_CONTRACTID);
-	nsCOMPtr<nsIPrefBranch> pref;
-	prefService->GetBranch ("", getter_AddRefs (pref));
+	nsresult rv;
+	nsCOMPtr<nsIPrefService> prefService (do_GetService (NS_PREFSERVICE_CONTRACTID, &rv));
+	NS_ENSURE_SUCCESS (rv, FALSE);
 
-	if (pref) {
-		nsresult rv = pref->SetCharPref (key, value);
-		return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-	}
-	
-	return FALSE;
+	nsCOMPtr<nsIPrefBranch> pref;
+	rv = prefService->GetBranch ("", getter_AddRefs (pref));
+	NS_ENSURE_SUCCESS (rv, FALSE);
+
+	rv = pref->SetCharPref (key, value);
+
+	return NS_SUCCEEDED (rv) != PR_FALSE;
 }
 
 static gboolean
 gecko_prefs_set_int (const gchar *key, gint value)
 {
-	nsCOMPtr<nsIPrefService> prefService =
-		do_GetService (NS_PREFSERVICE_CONTRACTID);
-	nsCOMPtr<nsIPrefBranch> pref;
-	prefService->GetBranch ("", getter_AddRefs (pref));
+	nsresult rv;
+	nsCOMPtr<nsIPrefService> prefService (do_GetService (NS_PREFSERVICE_CONTRACTID, &rv));
+	NS_ENSURE_SUCCESS (rv, FALSE);
 
-	if (pref) {
-		nsresult rv = pref->SetIntPref (key, value);
-		return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-	}
-	
-	return FALSE;
+	nsCOMPtr<nsIPrefBranch> pref;
+	rv = prefService->GetBranch ("", getter_AddRefs (pref));
+	NS_ENSURE_SUCCESS (rv, FALSE);
+
+	rv = pref->SetIntPref (key, value);
+
+	return NS_SUCCEEDED (rv) != PR_FALSE;
 }
 
 extern "C" void
@@ -124,6 +124,8 @@ yelp_gecko_set_color (YelpColorType type, const gchar *color)
 		gecko_prefs_set_string ("browser.anchor_color",
 					color);
 		break;
+	default:
+		break;
 	}
 }
 
@@ -131,7 +133,7 @@ extern "C" void
 yelp_gecko_set_font (YelpFontType font_type, const gchar *fontname)
 {
 	gchar *name;
-	gint   size;
+	gint   size = 0;
 
 	name = NULL;
 	if (!yelp_util_split_font_string (fontname, &name, &size)) {
@@ -160,6 +162,8 @@ yelp_gecko_set_font (YelpFontType font_type, const gchar *fontname)
 		gecko_prefs_set_int ("font.size.fixed.x-western", 
 				     size);
 		break;
+	default:
+		break;
 	}
 
 	g_free (name);
