@@ -115,8 +115,6 @@ static void        window_handle_page             (YelpWindow        *window,
 static void        window_disconnect              (YelpWindow        *window);
 
 /** Window Callbacks **/
-static void        yelp_window_destroyed          (GtkWidget         *window,
-						   gpointer           user_data);
 static gboolean    window_configure_cb            (GtkWidget         *widget,
 						   GdkEventConfigure *event,
 						   gpointer           data);
@@ -139,6 +137,10 @@ static void        html_uri_selected_cb           (YelpHtml          *html,
 						   gchar             *uri,
 						   gboolean           handled,
 						   gpointer           user_data);
+static gboolean    html_frame_selected_cb         (YelpHtml          *html,
+						   gchar             *uri,
+						   gboolean           handled,
+						   gpointer           user_data);
 static void        html_title_changed_cb          (YelpHtml          *html,
 						   gchar             *title,
 						   gpointer           user_data);
@@ -217,6 +219,8 @@ static void        window_find_clicked_cb         (GtkWidget         *button,
 						   YelpWindow        *window);
 static void        window_find_next_cb            (GtkAction *action, 
 						   YelpWindow *window);
+static void        window_find_previous_cb        (GtkAction *action, 
+						   YelpWindow *window);
 static gboolean    tree_model_iter_following      (GtkTreeModel      *model,
 						   GtkTreeIter       *iter);
 
@@ -244,6 +248,8 @@ struct _YelpWindowPriv {
     GtkToolItem    *find_prev;
     GtkToolItem    *find_next;
     gchar          *find_string;
+    GtkAction      *find_next_menu;
+    GtkAction      *find_prev_menu;
 
     /* Open Location */
     GtkWidget      *location_dialog;
@@ -355,10 +361,15 @@ static const GtkActionEntry entries[] = {
       "<Control>F",
       NULL,
       G_CALLBACK (window_find_cb) },
+    { "FindPrev", NULL,
+      N_("Find Pre_vious"),
+      "<Control><shift>G",
+      N_("Find previous occurrence of the word or phrase"),
+      G_CALLBACK (window_find_previous_cb) },
     { "FindNext", NULL,
       N_("Find Ne_xt"),
       "<Control>G",
-      NULL,
+      N_("Find next occurrence of the word or phrase"),
       G_CALLBACK (window_find_next_cb) },
     { "Preferences", GTK_STOCK_PREFERENCES,
       N_("_Preferences"),
@@ -491,6 +502,18 @@ window_init (YelpWindow *window)
 		      NULL);
 
     gtk_window_set_title (GTK_WINDOW (window), _("Help Browser"));
+   
+    window_populate (window);
+}
+
+static void
+window_dispose (GObject *object)
+{
+	YelpWindow *window = YELP_WINDOW (object);
+
+	window_disconnect (YELP_WINDOW (window));
+
+	parent_class->dispose (object);
 }
 
 static void
@@ -504,6 +527,10 @@ window_finalize (GObject *object)
 
     g_free (priv->find_string);
     
+    if (priv->current_doc)
+	    yelp_doc_info_unref (priv->current_doc);
+    g_free (priv->current_frag);
+
     /* FIXME there are many more things to free */
 
     parent_class->finalize (object);
@@ -516,6 +543,7 @@ window_class_init (YelpWindowClass *klass)
 
     parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
 
+    object_class->dispose = window_dispose;
     object_class->finalize = window_finalize;
 
     signals[NEW_WINDOW_REQUESTED] =
@@ -713,22 +741,7 @@ load_data_free (YelpLoadData *data)
 GtkWidget *
 yelp_window_new (GNode *doc_tree, GList *index)
 {
-    YelpWindow     *window;
-    YelpWindowPriv *priv;
-
-    window = g_object_new (YELP_TYPE_WINDOW, NULL);
-
-    priv   = window->priv;
-
-    d (g_print ("yelp_window_new\n"));
-
-    window_populate (window);
-
-    g_signal_connect (G_OBJECT (window), "destroy",
-		      G_CALLBACK (yelp_window_destroyed),
-		      NULL);
-
-    return GTK_WIDGET (window);
+    return GTK_WIDGET (g_object_new (YELP_TYPE_WINDOW, NULL));
 }
 
 void
@@ -738,13 +751,18 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
     YelpDocInfo    *doc_info;
     gchar          *frag_id;
     GtkAction      *action;
-
+    gchar          *real_uri;
     g_return_if_fail (YELP_IS_WINDOW (window));
 
+    if (g_str_has_prefix (uri, "info:") && g_str_has_suffix (uri, "dir")) {
+	real_uri = g_strdup ("x-yelp-toc:#Info");
+    } else {
+	real_uri = g_strdup (uri);
+    }
     d (g_print ("yelp_window_load\n"));
     d (g_print ("  uri = \"%s\"\n", uri));
 
-    if (!uri) {
+    if (!real_uri) {
 	GError *error = NULL;
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_NO_DOC,
 		     _("The Uniform Resource Identifier for the file is "
@@ -755,17 +773,17 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 
     priv = window->priv;
 
-    doc_info = yelp_doc_info_get (uri, FALSE);
+    doc_info = yelp_doc_info_get (real_uri, FALSE);
     if (!doc_info) {
 	GError *error = NULL;
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_NO_DOC,
 		     _("The Uniform Resource Identifier ‘%s’ is invalid "
 		       "or does not point to an actual file."),
-		     uri);
+		     real_uri);
 	window_error (window, error, FALSE);
 	return;
     }
-    frag_id  = yelp_uri_get_fragment (uri);
+    frag_id  = yelp_uri_get_fragment (real_uri);
 
     if (priv->current_doc && yelp_doc_info_equal (priv->current_doc, doc_info)) {
 	if (priv->current_frag) {
@@ -798,6 +816,7 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 
     if (priv->current_frag != frag_id)
 	g_free (frag_id);
+    g_free (real_uri);
 }
 
 YelpDocInfo *
@@ -822,9 +841,7 @@ window_do_load (YelpWindow  *window,
 		gchar       *frag_id)
 {
     GtkAction      *action;
-    YelpWindowPriv *priv;
     GError         *error = NULL;
-    gboolean        handled = FALSE;
     gchar          *uri;
 
     g_return_if_fail (YELP_IS_WINDOW (window));
@@ -832,9 +849,7 @@ window_do_load (YelpWindow  *window,
 
     d (g_print ("window_do_laod\n"));
 
-    priv = window->priv;
-
-    action = gtk_action_group_get_action (priv->action_group, 
+    action = gtk_action_group_get_action (window->priv->action_group, 
 					  "PrintDocument");
     g_object_set (G_OBJECT (action),
 		  "sensitive",
@@ -844,7 +859,7 @@ window_do_load (YelpWindow  *window,
     switch (yelp_doc_info_get_type (doc_info)) {
     case YELP_DOC_TYPE_MAN:
 #ifdef ENABLE_MAN
-	handled = window_do_load_pager (window, doc_info, frag_id);
+	window_do_load_pager (window, doc_info, frag_id);
 	break;
 #else
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
@@ -854,7 +869,7 @@ window_do_load (YelpWindow  *window,
 
     case YELP_DOC_TYPE_INFO:
 #ifdef ENABLE_INFO
-	handled = window_do_load_pager (window, doc_info, frag_id);
+	window_do_load_pager (window, doc_info, frag_id);
 	break;
 #else
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
@@ -868,11 +883,11 @@ window_do_load (YelpWindow  *window,
 		      TRUE,
 		      NULL);
     case YELP_DOC_TYPE_TOC:
-	handled = window_do_load_pager (window, doc_info, frag_id);
+	window_do_load_pager (window, doc_info, frag_id);
 	break;
     case YELP_DOC_TYPE_SEARCH:
 #ifdef ENABLE_SEARCH
-	handled = window_do_load_pager (window, doc_info, frag_id);
+	window_do_load_pager (window, doc_info, frag_id);
 	break;
 #else
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
@@ -886,7 +901,7 @@ window_do_load (YelpWindow  *window,
 	break;
     case YELP_DOC_TYPE_HTML:
     case YELP_DOC_TYPE_XHTML:
-	handled = window_do_load_html (window, doc_info, frag_id);
+	window_do_load_html (window, doc_info, frag_id);
 	break;
     case YELP_DOC_TYPE_EXTERNAL:
 	history_step_back (window);
@@ -1007,7 +1022,7 @@ window_populate (YelpWindow *window)
     
 #ifdef ENABLE_SEARCH
     action =  gtk_entry_action_new ("Search",
-				    _("Search"),
+				    _("_Search:"),
 				    _("Search for other documentation"),
 				    NULL);
     g_signal_connect (G_OBJECT (action), "activate",
@@ -1050,21 +1065,20 @@ window_populate (YelpWindow *window)
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 
-    action = gtk_action_group_get_action (priv->action_group, 
-						  "PrintDocument");
-    g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
-
     priv->popup = gtk_ui_manager_get_widget(priv->ui_manager, "ui/main_popup");
     priv->maillink = gtk_ui_manager_get_widget(priv->ui_manager, "ui/mail_popup");
     priv->merge_id = gtk_ui_manager_new_merge_id (priv->ui_manager);
 
+    priv->find_next_menu = gtk_action_group_get_action (priv->action_group, 
+							"FindNext");
+    priv->find_prev_menu = gtk_action_group_get_action (priv->action_group, 
+						      "FindPrev");
+
     priv->pane = gtk_hpaned_new ();
-    gtk_widget_ref (priv->pane);
     /* We should probably remember the last position and set to that. */
     gtk_paned_set_position (GTK_PANED (priv->pane), 180);
 
     priv->side_sw = gtk_scrolled_window_new (NULL, NULL);
-    gtk_widget_ref (priv->side_sw);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->side_sw),
 				    GTK_POLICY_AUTOMATIC,
 				    GTK_POLICY_AUTOMATIC);
@@ -1115,6 +1129,10 @@ window_populate (YelpWindow *window)
 		      "uri_selected",
 		      G_CALLBACK (html_uri_selected_cb),
 		      window);
+    g_signal_connect (priv->html_view,
+		      "frame_selected",
+		      G_CALLBACK (html_frame_selected_cb),
+		      window);
     g_signal_connect (priv->html_view,
 		      "title_changed",
 		      G_CALLBACK (html_title_changed_cb),
@@ -1151,7 +1169,7 @@ window_populate_find (YelpWindow *window, GtkWidget *find_bar)
     g_return_if_fail (GTK_IS_TOOLBAR (find_bar));
 
     box = gtk_hbox_new (FALSE, 0);
-    label = gtk_label_new_with_mnemonic (_("Find:"));
+    label = gtk_label_new_with_mnemonic (_("Fin_d:"));
     gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 6);
 
     priv->find_entry = gtk_entry_new ();
@@ -1318,7 +1336,6 @@ window_do_load_pager (YelpWindow  *window,
 	    window_error (window, error, TRUE);
 	handled = FALSE;
 	goto done;
-	break;
     case YELP_PAGER_STATE_RUNNING:
     case YELP_PAGER_STATE_FINISHED:
 	/* Check if the page exists */
@@ -1466,6 +1483,13 @@ window_do_load_html (YelpWindow    *window,
 
     while ((result = gnome_vfs_read
 	    (handle, buffer, BUFFER_SIZE, &n)) == GNOME_VFS_OK) {
+	gchar *tmp;
+
+	tmp = g_utf8_strup (buffer, n);
+	if (strstr (tmp, "<FRAMESET"))
+	    yelp_html_frames (priv->html_view, TRUE);
+	g_free (tmp);
+
 	yelp_html_write (priv->html_view, buffer, n);
     }
 
@@ -1692,26 +1716,6 @@ window_disconnect (YelpWindow *window)
 
 /** Window Callbacks **********************************************************/
 
-static void
-yelp_window_destroyed (GtkWidget *window,
-		       gpointer   user_data)
-{
-    YelpWindowPriv *priv;
-
-    g_return_if_fail (YELP_IS_WINDOW (window));
-
-    priv = YELP_WINDOW(window)->priv;
-
-    window_disconnect (YELP_WINDOW (window));
-
-    if (priv->current_doc)
-	yelp_doc_info_unref (priv->current_doc);
-    g_free (priv->current_frag);
-
-    g_object_unref (priv->pane);
-    g_object_unref (priv->side_sw);
-}
-
 static gboolean
 window_configure_cb (GtkWidget         *widget,
 		     GdkEventConfigure *event,
@@ -1846,6 +1850,30 @@ html_uri_selected_cb (YelpHtml  *html,
     }
 }
 
+static gboolean
+html_frame_selected_cb (YelpHtml *html, gchar *uri, gboolean handled,
+			gpointer user_data)
+{
+    YelpWindow *window = YELP_WINDOW (user_data);
+    gboolean handle;
+    YelpDocInfo *info = yelp_doc_info_get (uri, FALSE);
+    switch (yelp_doc_info_get_type (info)) {
+    case YELP_DOC_TYPE_HTML:
+    case YELP_DOC_TYPE_XHTML:
+	handle = TRUE;
+	break;
+    default:
+	handle = FALSE;
+	break;
+    }
+    if (handle) {
+	return FALSE;
+    } else {
+	yelp_window_load (window, uri);
+    }
+    return TRUE;
+}
+
 static void
 html_title_changed_cb (YelpHtml  *html,
 		       gchar     *title,
@@ -2562,14 +2590,14 @@ static void window_copy_link_cb (GtkAction *action, YelpWindow *window)
 			    window->priv->uri,
 			    -1);
     g_free (window->priv->uri);
-};
+}
 
 static void
 window_open_link_cb (GtkAction *action, YelpWindow *window)
 {
     yelp_window_load (window, window->priv->uri);
     g_free (window->priv->uri);
-};
+}
 
 static void
 window_open_link_new_cb (GtkAction *action, YelpWindow *window)
@@ -2577,7 +2605,7 @@ window_open_link_new_cb (GtkAction *action, YelpWindow *window)
     g_signal_emit (window, signals[NEW_WINDOW_REQUESTED], 0,
 		   window->priv->uri);
     g_free (window->priv->uri);
-};
+}
 
 static void
 window_copy_mail_cb (GtkAction *action, YelpWindow *window)
@@ -2637,14 +2665,14 @@ location_response_cb (GtkDialog *dialog, gint id, YelpWindow *window)
     d (g_print ("  id = %i\n", id));
 
     if (id == GTK_RESPONSE_OK) {
-	const gchar *uri =
+	const gchar *uri = 
 	    gtk_entry_get_text (GTK_ENTRY (window->priv->location_entry));
+    
 	yelp_window_load (window, uri);
     }
 
     window->priv->location_dialog = NULL;
     window->priv->location_entry  = NULL;
-
     gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
@@ -2691,6 +2719,24 @@ window_find_again (YelpWindow *window, gboolean forward)
     return yelp_html_find_again (priv->html_view, forward);
 }
 
+static void
+window_find_previous_cb (GtkAction *action, YelpWindow *window)
+{
+    YelpWindowPriv * priv;
+
+    priv = window->priv;
+    if (!window_find_again (window, FALSE)) {
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), TRUE);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), FALSE);
+	g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", TRUE, 
+		      NULL);
+	g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", FALSE, 
+		      NULL);
+    } else {
+	window_find_buttons_set_sensitive (window, TRUE);
+    }
+}
+
 static void
 window_find_next_cb (GtkAction *action, YelpWindow *window)
 {
@@ -2700,6 +2746,10 @@ window_find_next_cb (GtkAction *action, YelpWindow *window)
     if (!window_find_again (window, TRUE)) {
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), FALSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), TRUE);
+	g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", FALSE, 
+		      NULL);
+	g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", TRUE, 
+		      NULL);
     } else {
 	window_find_buttons_set_sensitive (window, TRUE);
     }
@@ -2719,6 +2769,10 @@ window_find_clicked_cb (GtkWidget  *widget,
 	if (!window_find_again (window, TRUE)) {
 	    gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), FALSE);
 	    gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), TRUE);
+	    g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", FALSE, 
+			  NULL);
+	    g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", TRUE, 
+			  NULL);
 	} else {
 	    window_find_buttons_set_sensitive (window, TRUE);
 	}
@@ -2727,6 +2781,10 @@ window_find_clicked_cb (GtkWidget  *widget,
 	if (!window_find_again (window, FALSE)) {
 	    gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), TRUE);
 	    gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), FALSE);
+	    g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", TRUE, 
+			  NULL);
+	    g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", FALSE, 
+			  NULL);
 	} else {
 	    window_find_buttons_set_sensitive (window, TRUE);
 	}
@@ -2759,6 +2817,10 @@ window_find_entry_changed_cb (GtkEditable *editable,
 	if (found) {
 	    gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), TRUE);
 	    gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), FALSE);
+	    g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", TRUE, 
+			  NULL);
+	    g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", FALSE, 
+			  NULL);
 	} else {
 	    window_find_buttons_set_sensitive (window, TRUE);
 	}
@@ -2784,6 +2846,10 @@ window_find_entry_activate_cb (GtkEditable *editable,
     if (!window_find_again (window, TRUE)) {
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), FALSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), TRUE);
+	g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", FALSE, 
+		      NULL);
+	g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", TRUE, 
+		      NULL);
     } else {
 	window_find_buttons_set_sensitive (window, TRUE);
     }
@@ -2823,6 +2889,10 @@ window_find_buttons_set_sensitive (YelpWindow  *window,
 
     gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), sensitive);
     gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), sensitive);
+    g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", sensitive, 
+		  NULL);
+    g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", sensitive, 
+		  NULL);
 }
 
 static gboolean
